/************************************************************************/
/* Author: 	Hirotaka Miura.						*/
/* Contact: 	Hirotaka.Miura@gmail.com.				*/
/************************************************************************/

/************************************************************************/
/* 05/07/2011: 	Modified.						*/
/* 05/02/2011: 	Previouly modified.					*/
/* 04/17/2011: 	Created.						*/
/* Program: 	Postcomputation command.				*/
/************************************************************************/
/* Declare program and Stata version.					*/
program netsummarize, sortpreserve eclass
version 11.1
/************************************************************************/
/****** 		DEFINE SYNTAX				    *****/
/************************************************************************/
#d;
  syntax anything(id="mata matrix name" name=mname) /* Mata matrix.	*/
    , /* By default, summary statistics set to rows.			*/
    Generate(name) /* New variables to generate for source and target.	*/
    Statistic(string)
; #d cr
/************************************************************************/
/*****		PARSE SYNTAX AND ERROR HANDLE			    *****/
/************************************************************************/
/***** Error if last results not found.					*/
capture assert(e(cmd)=="network")
if _rc!=0{
  di as error "last results not found"
  exit 301
}
/***** Error if invalid summary statistic specified.			*/
capture _statistic_parse , `statistic'
if _rc!=0{
  di as error "invalid summary statistic"
  exit 198
}
/***** Error if specified matrix does not exist in Mata.		*/
capture mata sizeof(`mname')
if _rc!=0{
  di as error "`mname' not found in Mata"
  exit 111
}
/***** Error if specified variables to generate already exists in Stata.*/
confirm new var `generate'_source `generate'_target
qui gen `generate'_source=.
qui gen `generate'_target=. 
label var `generate'_source "`statistic' of Mata matrix `mname'"
label var `generate'_target "`statistic' of Mata matrix `mname'"
/************************************************************************/
/***** 			RETRIEVE STATISTIC			    *****/
/************************************************************************/
/***** Generate temporary 'touse' sample marker variable.		*/
tempvar touse
gen `touse'=0
qui replace `touse'=1 if e(sample)
/***** Obtain statistics.						*/
tempname rc
mata: `rc'=main(`mname')
mata: st_local("_rc",strofreal(`rc'))
/***** Exit if internal error occurs in main().				*/
if `_rc'!=0{
  /* Clear any save values.						*/
  mata st_eclear(); st_sclear(); st_rclear()
  /* Harmonize Mata and Stata error codes if necessary.			*/
  if `_rc'==3200 error(503)
  else error(`_rc')
}
/***** Repost e(sample).						*/
ereturn repost, esample(`touse')
end

/************************************************************************/
/***** 			MATA SUBROUTINE			            *****/
/************************************************************************/
/* Initialize Mata environment.						*/
mata:
/* Set variable declarations on.					*/
mata set matastrict on
mata set matalnum off
/* Declare function.							*/
real scalar main(real matrix X){
  /* Declare local variables.						*/
  transmorphic matrix V /* Varlist matrix.				*/
  real matrix E /* Edge list with edge weights.				*/    
  real matrix RES /* Results matrix.					*/
  real vector o /* Unique originial ID list.				*/  
  real vector res /* Results vector.					*/
  real scalar n /* Network size.					*/
  real scalar i,k /* Row counters.					*/
  string scalar c /* Stata command string.				*/
  string scalar s /* Source vertices variable to hold results.		*/
  string scalar t /* Target vertices variable to hold results.		*/
  string scalar u /* Summary statistic to compute.			*/
  /**********************************************************************/  
  /***** Preparation for generating edge list.				*/
  /**********************************************************************/  
  /***** Retrive varlist based on e(sample).				*/
  /* Numeric varlist.							*/
  if(st_isnumvar(st_global("e(source)"))){
    V=st_data(.,st_global("e(source)")+" "+st_global("e(target)"),
      st_local("touse"))
  }
  /* String varlist.							*/
  else{
    V=st_sdata(.,st_global("e(source)")+" "+st_global("e(target)"),
      st_local("touse"))
  }
  /**********************************************************************/
  /***** Generate edge list and make adjustments.			*/
  /**********************************************************************/
  /***** Value-label mapping.						*/
  /* Convert original IDs to string if numeric.				*/
  if(!isstring(V)) V=strofreal(V,"%200.0g")  
  /* Obtain a list of unique original IDs.				*/
  o=uniqrows((V[.,1]\V[.,2]))
  /* Create value-label.						*/
  st_vlmodify("value_label",(1::rows(o)),o)
  /***** Generate edge list.						*/
  E=J(rows(V),2,.)
  E=st_vlsearch("value_label",V) 
  /**********************************************************************/
  /***** Compute statistics.						*/
  /**********************************************************************/  
  /* Obtain network size.						*/
  n=st_numscalar("e(vertices)")       
  /* Check dimensions of specified matrix inputted as argument.		*/
  /* Matrix may be nxn, nx1, or 1x1.					*/
  if(length(X)!=n*n & length(X)!=n & length(X)!=1){
    /* Return conformability error code if wrong number of elements.	*/
    return(3200)
  }  
  /***** Compute summary statistics.					*/
  /* Convert to column vector if necessary. 				*/
  u=st_local("statistic")
  if(u=="mean") 	res=mean(mean(X)')
  if(u=="min") 		res=min(X)
  if(u=="max") 		res=max(X)
  if(u=="sum") 		res=sum(X)
  if(u=="missing") 	res=missing(X)
  if(u=="nonmissing") 	res=nonmissing(X)
  if(u=="rowmean") 	res=mean(X')'
  if(u=="rowmin")	res=rowmin(X)
  if(u=="rowmax")	res=rowmax(X)
  if(u=="rowsum")	res=rowsum(X)
  if(u=="rowmissing")	res=rowmissing(X)
  if(u=="rownonmissing") res=rownonmissing(X)
  if(u=="colmean")	res=mean(X)'
  if(u=="colmin")	res=colmin(X)'
  if(u=="colmax")	res=colmax(X)'
  if(u=="colsum")	res=colsum(X)'
  if(u=="colmissing")	res=colmissing(X)'
  if(u=="colnonmissing") res=colnonmissing(X)'
  /* If result is a scalar, convert to nx1 vector.			*/
  if(length(res)==1) res=J(n,1,res)
  /***** Generate results matrix based on edge list.			*/
  /* Declare empty results matrix.					*/
  RES=J(rows(E),2,.)
  /* Loop through rows of results vector.				*/
  for(i=1;i<=rows(res);i++){
    /* Loop through rows of edge list.					*/
    for(k=1;k<=rows(E);k++){
      /* Fill in first column in indices match.				*/
      if(E[k,1]==i) RES[k,1]=res[i]
      /* Fill in second column in indices match.			*/
      if(E[k,2]==i) RES[k,2]=res[i]
    }
  }
  /***** Return results to Stata.					*/
  s=st_local("generate")+"_source"
  t=st_local("generate")+"_target"
  st_store(.,(s,t),st_local("touse"),RES)
  /* Success.								*/
  return(0)
}
end

/************************************************************************/
/* Subroutine to parse statistic option.				*/
/************************************************************************/
program define _statistic_parse, sclass
  #d;
  syntax [anything] 
  [, 
  mean /* Mean - not calculated if any entries contain missing.		*/
  min /* Minimum - missing treated as zero.				*/
  max /* Maximum - missing treated as zero.				*/
  sum /* Sum - missing treated as zero.					*/
  missing /* Missing - count of missing values.				*/
  nonmissing /* Nonmissing - count of nonmissing values.		*/
  rowmean
  rowmin
  rowmax
  rowsum
  rowmissing
  rownonmissing
  colmean
  colmin
  colmax
  colsum
  colmissing
  colnonmissing  
  ]
  ; #d cr
end


